<?php

namespace Celebrio\FileSystem;

use Celebrio\FileSystem\IVirtualFileSystem;
use Celebrio\FileSystem\VirtualFileSystem;
use Celebrio\FileSystem\DataFile;
use Celebrio\Core\ServiceProvider;
use Celebrio\Modules;

use Celebrio\Diagnostics\Timer;
use Celebrio\Diagnostics\TimerLevel;

use Nette\Object;

/**
 *
 * @author Petr, pavel, Honza
 */
//TODO: add security control of api only for users applications!
class FileSystemApi extends Object implements IFileSystem {

    /**
     * @var IVirtualFileSystem
     */
    private $fileSystemManager;

    /**
     * Sets the attribute $userId according to the given parameter.
     *
     * @param \Nette\Web\User $user the object containing the user ID
     *        or the ID itself.
     */
    public function __construct($user = null) {
        Timer::start("FSAPI - init VFS",TimerLevel::OVERVIEW);
        $this->setFileSystemManager(new VirtualFileSystem($user));
        Timer::stop("FSAPI - init VFS");
    }

    public function getFileSystemManager() {
        return $this->fileSystemManager;
    }

    public function setFileSystemManager($fsm) {
        $this->fileSystemManager = $fsm;
    }

    public function load($urlFrom, $stream = true) {
        Timer::start("FSAPI - load", TimerLevel::OVERVIEW);
        $dataFile = $this->makeDataFile($urlFrom);
        $loaded = $this->getFileSystemManager()->load($dataFile, $stream);
        if ($loaded->isDirectory()) {
            Timer::stop("FSAPI - load");
            return $loaded;
        } else {
            if ($stream) {
                header('Content-type: '. $dataFile->getMimetypeText());
                header("Pragma: private");
                header("Cache-Control: private, max-age:3600"); // hold in chache for an hour
                header("Content-Transfer-Encoding: binary\n");
                $handler = $loaded->getData();
                while (!feof($handler)) {
                    echo fread($handler, 1 * (1024 * 1024));
                    ob_flush();
                    flush();
                }
                fclose($handler);
                Timer::stop("FSAPI - load");
                Timer::stop("Application run"); // TODO: why this isnt loged?
				Timer::stop($_SERVER["REQUEST_URI"]); // TODO: why this isnt loged?
                exit;
            } else {
                Timer::stop("FSAPI - load");
                return $loaded;
            }
        }
        return $loaded;
    }

    public function save($urlFrom, $data, $mimeType = null, $storageName = null) {
        Timer::start("FSAPI - save", TimerLevel::OVERVIEW);
        if ($mimeType == null) {
            $mimeType = (empty($data) ? DataFile::DIR : "plain/text");
        }
        $dataFile = $this->makeDataFile($urlFrom);
        $dataFile->setData($data);
        $dataFile->setMimetypeText($mimeType);
        $dataFile->setFstorageName($this->resolveFileStorage($storageName));
        $this->getFileSystemManager()->save($dataFile);
        Timer::stop("FSAPI - save");
    }

    public function getFileInfo($urlFrom) {
        $dataFile = $this->makeDataFile($urlFrom);
        return $this->getFileSystemManager()->getFileInfo($dataFile);
    }

    public function copy($urlFrom, $urlTo, $rewrite) {
        $boolRewrite = $this->checkRewrite($rewrite);
        $dataFileFrom = $this->makeDataFile($urlFrom);
        $dataFileTo = $this->makeDataFile($urlTo);
        $driver = ServiceProvider::getService(Modules::FS_DRIVERS);
        $dataFileTo->setFstorageName($driver::NAME);
        $this->getFileSystemManager()->copy($dataFileFrom, $dataFileTo, $rewrite);
    }

    public function move($urlFrom, $urlTo, $rewrite) {
        $boolRewrite = $this->checkRewrite($rewrite);
        $dataFileFrom = $this->makeDataFile($urlFrom);
        $dataFileTo = $this->makeDataFile($urlTo);
        $this->getFileSystemManager()->move($dataFileFrom, $dataFileTo, $rewrite);
    }

    public function delete($urlFrom) {
        $dataFile = $this->makeDataFile($urlFrom);
        $dataFile->setFstorageName($this->resolveFileStorage());
        $this->getFileSystemManager()->delete($dataFile);
    }

    private function makeDataFile($url) {
        Timer::start("FSAPI - create dataFile", TimerLevel::OVERVIEW);
        $url = rtrim($url, '/');
        if (empty($url)) {
            $fileName = "/";
            $fileUrl = NULL;
        } else {
            $lastOccur = strrpos($url, '/') + 1;
            $fileUrl = substr($url, 0, $lastOccur);
            $fileName = substr($url, $lastOccur);
        }
        $dataFile = new DataFile($fileName, $fileUrl);
        Timer::stop("FSAPI - create dataFile");
        return $dataFile;
    }

    /**
     * just because of PHP crappy typecasting
     *
     * @param mixed $rewrite
     * @return boolean
     */
    private function checkRewrite($rewrite) {
        if (is_null($rewrite)) {
            return false;
        } elseif (is_bool($rewrite)) {
            return $rewrite;
        } elseif (is_string($rewrite)) {
            if ($rewrite === "true" || $rewrite == "1") {
                return true;
            } elseif ($rewrite === "false" || $rewrite == "0") {
                return false;
            }
        }
        throw new \InvalidArgumentException("Rewrite is not boolean!");
    }
    
    private function resolveFileStorage($storageName = null) {
        if($storageName == null) {
            $storage = ServiceProvider::getAllServiceNames(Modules::FS_DRIVERS);
            $storageName = $storage[0]::NAME;
        }
        return $storageName;
    }

}